@@ -58,12 +58,12 @@ class SpoofResponse:
                 u'%s Server Error: %s for url: %s' % (self.status_code, self.reason, self.url), response=self)
 
 # Relationship between py4cytoscape execution environment and Cytoscape execution environment
-from enum import Enum, auto
-class ExecutionEnvironment(Enum):
-    UNKNOWN = auto()
-    SHARED_WORKSTATION = auto()
-    REMOTE_DIRECT_URL = auto()
-    REMOTE_JUPYTER_BRIDGE = auto()
+#from enum import Enum, auto  # This causes major build problems, so leave it out
+class ExecutionEnvironment(): # (Enum):
+    UNKNOWN = 0 # auto()
+    SHARED_WORKSTATION = 1 # auto()
+    REMOTE_DIRECT_URL = 2 # auto()
+    REMOTE_JUPYTER_BRIDGE = 3 # auto()
 
 # Create a unique channel that identifies this process so other processes don't mix up messages
 _CHANNEL = None
@@ -222,10 +222,10 @@ def check_execution_environment(base_url):
             else:
                 _execution_environment = ExecutionEnvironment.REMOTE_DIRECT_URL
                 detail_logger.debug(f'Detected py4cytoscape running on Cytoscape workstation at {base_url}')
-        except:
+        except Exception as e1:
             # Cytoscape doesn't appear to be reachable via URL, so try reaching a remote Cytoscape via Jupyter-bridge
             try:
-                detail_logger.debug(f'Attempting to connect to remote Cytoscape')
+                detail_logger.debug(f'Attempting to connect to remote Cytoscape because of error {_error_content(e1)}')
                 do_request_jupyter_bridge('GET', _CYREST_URL_V1, headers={'Content-Type': 'application/json'})
                 _execution_environment = ExecutionEnvironment.REMOTE_JUPYTER_BRIDGE
                 detail_logger.debug(f'Detected Cytoscape via Jupyter-Bridge')
