@@ -21,11 +21,13 @@ TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR TH
 
 # print(f'Starting {__name__} module')
 
+from os import environ
 
-CATCHUP_FILTER_SECS = 0 # 1
-MODEL_PROPAGATION_SECS = 0 #2
-CATCHUP_NETWORK_SECS = 4 # How long to sleep between network operation retries
-CATCHUP_NETWORK_TIMEOUT_SECS = 60 # How long to keep retrying network operation
+CATCHUP_FILTER_SECS = int(environ.get('PY4CYTOSCAPE_CATCHUP_FILTER_SECS', '0')) # '1'
+MODEL_PROPAGATION_SECS = int(environ.get('PY4CYTOSCAPE_MODEL_PROPAGATION_SECS', '2'))
+CATCHUP_NETWORK_SECS = int(environ.get('PY4CYTOSCAPE_CATCHUP_NETWORK_SECS', '4')) # How long to sleep between network operation retries
+CATCHUP_NETWORK_TIMEOUT_SECS = int(environ.get('PY4CYTOSCAPE_CATCHUP_NETWORK_TIMEOUT_SECS', '60')) # How long to keep retrying network operation
+CATCHUP_NETWORK_MERGE_SECS = int(environ.get('PY4CYTOSCAPE_CATCHUP_NETWORK_MERGE_SECS', '1')) # How long to sleep waiting for merge to complete Network table
 
 def set_catchup_filter_secs(delay_secs):
     global CATCHUP_FILTER_SECS
@@ -43,4 +45,8 @@ def set_catchup_network_timeout_secs(delay_secs):
     global CATCHUP_NETWORK_TIMEOUT_SECS
     CATCHUP_NETWORK_TIMEOUT_SECS = delay_secs
 
+def set_catchup_network_merge_secs(delay_secs):
+    global CATCHUP_NETWORK_MERGE_SECS
+    CATCHUP_NETWORK_MERGE_SECS = delay_secs
+
 
