@@ -34,11 +34,13 @@ from . import styles
 from . import style_dependencies
 from . import tables
 
+
 # Internal module convenience imports
 from .exceptions import CyError
 from .py4cytoscape_utils import *
-from .py4cytoscape_logger import cy_log
+from .py4cytoscape_logger import cy_log, show_error
 from .py4cytoscape_tuning import MODEL_PROPAGATION_SECS
+from .style_visual_props import *
 
 
 # ==============================================================================
@@ -77,15 +79,6 @@ def update_style_defaults(style_name, defaults, base_url=DEFAULT_BASE_URL):
     See Also:
         :meth:`map_visual_property`
     """
-    _PROPERTY_NAMES = {'EDGE_COLOR': 'EDGE_UNSELECTED_PAINT', 'EDGE_THICKNESS': 'EDGE_WIDTH',
-                       'NODE_BORDER_COLOR': 'NODE_BORDER_PAINT', 'NODE_BORDER_LINE_TYPE': 'NODE_BORDER_STROKE'}
-
-    def normalize_prop_name(prop_name):
-        # Convert white space to '_' and uppercase everything (e.g., 'edge color' -> 'EDGE_COLOR')
-        visual_prop_name = re.sub('\\s+', '_', prop_name).upper()
-        if visual_prop_name in _PROPERTY_NAMES: visual_prop_name = _PROPERTY_NAMES[visual_prop_name]
-        return visual_prop_name
-
     if style_name is None:
         style_name = 'default'
         narrate(f'style_name not specified, so updating "default" style.')
@@ -93,6 +86,10 @@ def update_style_defaults(style_name, defaults, base_url=DEFAULT_BASE_URL):
     # process visual property, including common alternatives for vp names :)
     def_list = [{'visualProperty': normalize_prop_name(prop), 'value': val} for prop, val in defaults.items()]
 
+    # Verify values and adjust them if necessary
+    for prop in def_list:
+        prop['value'] = _validate_prop_value(prop['visualProperty'], prop['value'])
+
     res = commands.cyrest_put(f'styles/{style_name}/defaults', body=def_list, base_url=base_url,
                               require_json=False)
     return res
@@ -126,6 +123,8 @@ def get_visual_property_default(property, style_name=None, base_url=DEFAULT_BASE
         style_name = 'default'
         narrate(f'style_name not specified, so accessing "default" style.')
 
+    property = normalize_prop_name(property)
+
     # TODO: Should the property name be mapped like in update_style_defaults?
     res = commands.cyrest_get(f'styles/{style_name}/defaults/{property}', base_url=base_url)
     return res['value']
@@ -152,6 +151,8 @@ def set_visual_property_default(style_string, style_name=None, base_url=DEFAULT_
     Examples:
         >>> set_visual_property_default({'visualProperty': 'EDGE_UNSELECTED_PAINT', 'value': '#CCCCCC'}, style_name='galFiltered Style')
         ''
+        >>> set_visual_property_default({'visualProperty': 'EDGE_UNSELECTED_PAINT', 'value': 'pink'}, style_name='galFiltered Style')
+        ''
         >>> set_visual_property_default({'visualProperty': 'EDGE_TARGET_ARROW_SHAPE', 'value': 'CIRCLE'})
         ''
     """
@@ -159,6 +160,10 @@ def set_visual_property_default(style_string, style_name=None, base_url=DEFAULT_
         style_name = 'default'
         narrate(f'style_name not specified, so updating "default" style.')
 
+    style_string['visualProperty'] = normalize_prop_name(style_string['visualProperty'])
+    # Verify value and adjust it if necessary
+    style_string['value'] = _validate_prop_value(style_string['visualProperty'], style_string['value'])
+
     # TODO: Should the property name be mapped like in update_style_defaults?
     res = commands.cyrest_put(f'styles/{style_name}/defaults', body=[style_string], base_url=base_url,
                               require_json=False)
@@ -198,9 +203,9 @@ def set_node_border_color_default(new_color, style_name=None, base_url=DEFAULT_B
         ''
         >>> set_node_border_color_default('#CCCCCC')
         ''
+        >>> set_node_border_color_default('red')
+        ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'NODE_BORDER_PAINT', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -230,8 +235,6 @@ def set_node_border_width_default(new_width, style_name=None, base_url=DEFAULT_B
         >>> set_node_border_width_default(10)
         ''
     """
-    verify_dimensions('width', new_width)
-
     style = {'visualProperty': 'NODE_BORDER_WIDTH', 'value': new_width}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -261,8 +264,6 @@ def set_node_border_opacity_default(new_opacity, style_name=None, base_url=DEFAU
         >>> set_node_border_opacity_default(10)
         ''
     """
-    verify_opacities(new_opacity)
-
     style = {'visualProperty': 'NODE_BORDER_TRANSPARENCY', 'value': new_opacity}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -291,9 +292,9 @@ def set_node_color_default(new_color, style_name=None, base_url=DEFAULT_BASE_URL
         ''
         >>> set_node_color_default('#CCCCCC')
         ''
+        >>> set_node_color_default('red')
+        ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'NODE_FILL_COLOR', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -341,6 +342,8 @@ def set_node_custom_bar_chart(columns, type='GROUPED', colors=None, range=None,
         ''
         >>> set_node_custom_bar_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['#FF00FF', '#00FF00'], slot=2, style_name='galFiltered Style')
         ''
+        >>> set_node_custom_bar_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['red', 'green'], axis_color='blue', slot=2, style_name='galFiltered Style')
+        ''
 
     See Also:
         :meth:`set_node_custom_position`, :meth:`remove_node_custom_graphics`
@@ -360,6 +363,9 @@ def set_node_custom_bar_chart(columns, type='GROUPED', colors=None, range=None,
             palette = cyPalette('rdbu')
             colors = [palette[index] for index in [1, 9]]
 
+    colors = verify_hex_colors(colors)
+    axis_color = verify_hex_color(axis_color)
+
     if range is None:
         cols = tables.get_table_columns(columns=columns, base_url=base_url)
         min = cols[columns].min().min()  # Make sure this works when column contains NANs
@@ -418,6 +424,8 @@ def set_node_custom_box_chart(columns, colors=None, range=None, orientation='VER
         ''
         >>> set_node_custom_box_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['#FF00FF', '#00FF00'], slot=2, style_name='galFiltered Style')
         ''
+        >>> set_node_custom_box_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['red', 'green'], axis_color='blue', slot=2, style_name='galFiltered Style')
+        ''
 
     See Also:
         :meth:`set_node_custom_position`, :meth:`remove_node_custom_graphics`
@@ -427,6 +435,9 @@ def set_node_custom_box_chart(columns, colors=None, range=None, orientation='VER
     if colors is None:
         colors = cyPalette('rdbu') * len(columns)
 
+    colors = verify_hex_colors(colors)
+    axis_color = verify_hex_color(axis_color)
+
     if range is None:
         cols = tables.get_table_columns(columns=columns, base_url=base_url)
         min = cols[columns].min().min()  # Make sure this works when column contains NANs
@@ -484,6 +495,8 @@ def set_node_custom_heat_map_chart(columns, colors=None, range=None, orientation
         ''
         >>> set_node_custom_heat_map_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['#123456', '#654321', '#112233', '#888888'], slot=2, style_name='galFiltered Style')
         ''
+        >>> set_node_custom_heat_map_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['red', 'green', 'pink', 'purple'], axis_color='blue', slot=2, style_name='galFiltered Style')
+        ''
 
     See Also:
         :meth:`set_node_custom_position`, :meth:`remove_node_custom_graphics`
@@ -495,6 +508,9 @@ def set_node_custom_heat_map_chart(columns, colors=None, range=None, orientation
         colors = [palette[index] for index in [2, 6, 10]]
         colors.append('#888888')
 
+    colors = verify_hex_colors(colors)
+    axis_color = verify_hex_color(axis_color)
+
     if range is None:
         cols = tables.get_table_columns(columns=columns, base_url=base_url)
         min = cols[columns].min().min()  # Make sure this works when column contains NANs
@@ -552,6 +568,8 @@ def set_node_custom_line_chart(columns, colors=None, range=None, line_width=1.0,
         ''
         >>> set_node_custom_line_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['#FF00FF', '#00FF00'], slot=2, style_name='galFiltered Style')
         ''
+        >>> set_node_custom_line_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['red', 'green'], axis_color='blue', slot=2, style_name='galFiltered Style')
+        ''
 
     See Also:
         :meth:`set_node_custom_position`, :meth:`remove_node_custom_graphics`
@@ -561,6 +579,9 @@ def set_node_custom_line_chart(columns, colors=None, range=None, line_width=1.0,
     if colors is None:
         colors = cyPalette('set1') * len(columns)
 
+    colors = verify_hex_colors(colors)
+    axis_color = verify_hex_color(axis_color)
+
     if range is None:
         cols = tables.get_table_columns(columns=columns, base_url=base_url)
         min = cols[columns].min().min()  # Make sure this works when column contains NANs
@@ -611,6 +632,8 @@ def set_node_custom_pie_chart(columns, colors=None, start_angle=0.0,
         ''
         >>> set_node_custom_pie_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['#FF00FF', '#00FF00'], slot=2, style_name='galFiltered Style')
         ''
+        >>> set_node_custom_pie_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['red', 'green'], slot=2, style_name='galFiltered Style')
+        ''
 
     See Also:
         :meth:`set_node_custom_position`, :meth:`remove_node_custom_graphics`
@@ -619,6 +642,9 @@ def set_node_custom_pie_chart(columns, colors=None, start_angle=0.0,
 
     if colors is None:
         colors = cyPalette('set1') * len(columns)
+
+    colors = verify_hex_colors(colors)
+
     chart = {'cy_colors': colors, 'cy_colorScheme': 'Custom', 'cy_dataColumns': columns,
              'cy_startAngle': start_angle}
 
@@ -660,6 +686,8 @@ def set_node_custom_ring_chart(columns, colors=None, start_angle=0.0, hole_size=
         ''
         >>> set_node_custom_ring_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['#FF00FF', '#00FF00'], slot=2, style_name='galFiltered Style')
         ''
+        >>> set_node_custom_ring_chart(['AverageShortestPath', 'BetweennessCentrality'], colors=['red', 'green'], slot=2, style_name='galFiltered Style')
+        ''
 
     See Also:
         :meth:`set_node_custom_position`, :meth:`remove_node_custom_graphics`
@@ -668,6 +696,9 @@ def set_node_custom_ring_chart(columns, colors=None, start_angle=0.0, hole_size=
 
     if colors is None:
         colors = cyPalette('set1') * len(columns)
+
+    colors = verify_hex_colors(colors)
+
     chart = {'cy_colors': colors, 'cy_colorScheme': 'Custom', 'cy_dataColumns': columns,
              'cy_startAngle': start_angle, 'cy_holeSize': hole_size}
 
@@ -706,11 +737,13 @@ def set_node_custom_linear_gradient(colors=['#DDDDDD', '#888888'], anchors=[0.0,
     Examples:
         >>> set_node_custom_linear_gradient(style_name='galFiltered Style')
         ''
-        >>> set_node_custom_linear_gradient(colors=['#FF00FF', '#00FF00'], angle=90.0, slot=2, style_name='galFiltered Style')
+        >>> set_node_custom_linear_gradient(colors=['red', 'green'], angle=90.0, slot=2, style_name='galFiltered Style')
         ''
     """
     verify_slot(slot)
 
+    colors = verify_hex_colors(colors)
+
     chart = {'cy_angle': angle, 'cy_gradientColors': colors, 'cy_gradientFractions': anchors}
     style_string = {'visualProperty': f'NODE_CUSTOMGRAPHICS_{slot}',
                     'value': 'org.cytoscape.LinearGradient:' + json.dumps(chart)}
@@ -751,9 +784,13 @@ def set_node_custom_radial_gradient(colors=['#DDDDDD', '#888888'], anchors=[0.0,
         ''
         >>> set_node_custom_radial_gradient(colors=['#FF00FF', '#00FF00'], x_center=0.0, y_center=0.0, slot=2, style_name='galFiltered Style')
         ''
+        >>> set_node_custom_radial_gradient(colors=['red', 'green'], x_center=0.0, y_center=0.0, slot=2, style_name='galFiltered Style')
+        ''
     """
     verify_slot(slot)
 
+    colors = verify_hex_colors(colors)
+
     chart = {'cy_gradientColors': colors, 'cy_gradientFractions': anchors, 'cy_center': {'x': x_center, 'y': y_center}}
     style_string = {'visualProperty': f'NODE_CUSTOMGRAPHICS_{slot}',
                     'value': 'org.cytoscape.RadialGradient:' + json.dumps(chart)}
@@ -862,8 +899,6 @@ def set_node_fill_opacity_default(new_opacity, style_name=None, base_url=DEFAULT
         >>> set_node_fill_opacity_default(10)
         ''
     """
-    verify_opacities(new_opacity)
-
     style = {'visualProperty': 'NODE_TRANSPARENCY', 'value': new_opacity}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -922,8 +957,6 @@ def set_node_font_size_default(new_size, style_name=None, base_url=DEFAULT_BASE_
         >>> set_node_font_size_default(20)
         ''
     """
-    verify_dimensions('size', new_size)
-
     style = {'visualProperty': 'NODE_LABEL_FONT_SIZE', 'value': new_size}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -953,7 +986,6 @@ def set_node_height_default(new_height, style_name=None, base_url=DEFAULT_BASE_U
         >>> set_node_height_default(20)
         ''
     """
-    verify_dimensions('height', new_height)
     # TODO: Shouldn't we be setting this back to its original value? ... and checking return result?
     style_dependencies.lock_node_dimensions(False, style_name=style_name, base_url=base_url)
 
@@ -1015,8 +1047,6 @@ def set_node_label_color_default(new_color, style_name=None, base_url=DEFAULT_BA
         >>> set_node_label_color_default('#CCCCCC')
         ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'NODE_LABEL_COLOR', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1046,8 +1076,6 @@ def set_node_label_opacity_default(new_opacity, style_name=None, base_url=DEFAUL
         >>> set_node_label_opacity_default(50)
         ''
     """
-    verify_opacities(new_opacity)
-
     style = {'visualProperty': 'NODE_LABEL_TRANSPARENCY', 'value': new_opacity}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1104,8 +1132,6 @@ def set_node_selection_color_default(new_color, style_name=None, base_url=DEFAUL
         >>> set_node_selection_color_default('#CCCCCC')
         ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'NODE_SELECTED_PAINT', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1135,13 +1161,9 @@ def set_node_shape_default(new_shape, style_name=None, base_url=DEFAULT_BASE_URL
         >>> set_node_shape_default('ELLIPSE')
         ''
     """
-    new_shape = new_shape.upper()
-    if new_shape in styles.get_node_shapes(base_url=base_url):
-        style = {'visualProperty': 'NODE_SHAPE', 'value': new_shape}
-        res = set_visual_property_default(style, style_name, base_url=base_url)
-        return res
-    else:
-        raise CyError(f'"{new_shape}" is not a valid shape. For valid ones, check get_node_shapes().')
+    style = {'visualProperty': 'NODE_SHAPE', 'value': new_shape}
+    res = set_visual_property_default(style, style_name, base_url=base_url)
+    return res
 
 
 @cy_log
@@ -1168,8 +1190,6 @@ def set_node_size_default(new_size, style_name=None, base_url=DEFAULT_BASE_URL):
         >>> set_node_size_default(20)
         ''
     """
-    verify_dimensions('size', new_size)
-
     # TODO: Shouldn't we be setting this back to its original value? ... and checking return result?
     style_dependencies.lock_node_dimensions(True, style_name=style_name, base_url=base_url)
 
@@ -1202,8 +1222,6 @@ def set_node_width_default(new_width, style_name=None, base_url=DEFAULT_BASE_URL
         >>> set_node_width_default(20)
         ''
     """
-    verify_dimensions('width', new_width)
-
     # TODO: Shouldn't we be setting this back to its original value? ... and checking return result?
     style_dependencies.lock_node_dimensions(False, style_name=style_name, base_url=base_url)
 
@@ -1271,8 +1289,6 @@ def set_edge_color_default(new_color, style_name=None, base_url=DEFAULT_BASE_URL
         >>> set_edge_color_default('#CCCCCC')
         ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'EDGE_UNSELECTED_PAINT', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     # TODO: Is it OK to lose the res value?
@@ -1335,8 +1351,6 @@ def set_edge_font_size_default(new_size, style_name=None, base_url=DEFAULT_BASE_
         >>> set_edge_font_size_default(20)
         ''
     """
-    verify_dimensions('size', new_size)
-
     style = {'visualProperty': 'EDGE_LABEL_FONT_SIZE', 'value': new_size}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1395,8 +1409,6 @@ def set_edge_label_color_default(new_color, style_name=None, base_url=DEFAULT_BA
         >>> set_edge_label_color_default('#CCCCCC')
         ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'EDGE_LABEL_COLOR', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1426,8 +1438,6 @@ def set_edge_label_opacity_default(new_opacity, style_name=None, base_url=DEFAUL
         >>> set_edge_label_opacity_default(10)
         ''
     """
-    verify_opacities(new_opacity)
-
     style = {'visualProperty': 'EDGE_LABEL_TRANSPARENCY', 'value': new_opacity}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1457,8 +1467,6 @@ def set_edge_line_width_default(new_width, style_name=None, base_url=DEFAULT_BAS
         >>> set_edge_line_width_default(10)
         ''
     """
-    verify_dimensions('width', new_width)
-
     style = {'visualProperty': 'EDGE_WIDTH', 'value': new_width}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1517,8 +1525,6 @@ def set_edge_opacity_default(new_opacity, style_name=None, base_url=DEFAULT_BASE
         >>> set_edge_opacity_default(10)
         ''
     """
-    verify_opacities(new_opacity)
-
     style = {'visualProperty': 'EDGE_TRANSPARENCY', 'value': new_opacity}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1577,8 +1583,6 @@ def set_edge_selection_color_default(new_color, style_name=None, base_url=DEFAUL
         >>> set_edge_selection_color_default('#CCCCCC')
         ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'EDGE_SELECTED_PAINT', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     # TODO: res is lost after this call
@@ -1613,8 +1617,6 @@ def set_edge_source_arrow_color_default(new_color, style_name=None, base_url=DEF
         >>> set_edge_source_arrow_color_default('#CCCCCC')
         ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'EDGE_SOURCE_ARROW_UNSELECTED_PAINT', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1644,8 +1646,6 @@ def set_edge_target_arrow_color_default(new_color, style_name=None, base_url=DEF
         >>> set_edge_target_arrow_color_default('#CCCCCC')
         ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'EDGE_TARGET_ARROW_UNSELECTED_PAINT', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
@@ -1794,9 +1794,45 @@ def set_background_color_default(new_color, style_name=None, base_url=DEFAULT_BA
         ''
         >>> set_background_color_default('#CCCCCC')
         ''
+        >>> set_background_color_default('red')
+        ''
     """
-    verify_hex_colors(new_color)
-
     style = {'visualProperty': 'NETWORK_BACKGROUND_PAINT', 'value': new_color}
     res = set_visual_property_default(style, style_name, base_url=base_url)
     return res
+
+def _validate_prop_value(prop, prop_val):
+    # Check property value to make sure it fits syntax
+
+    def none_check(value):
+        if value is None:
+            raise CyError(f'Property "{prop}" cannot be None. It must be a scalar.',
+                      caller=sys._getframe(4).f_code.co_name)
+        return value
+
+    if isinstance(prop_val, list):
+        raise CyError(f'Property "{prop}" cannot be a list ({prop_val}). It must be a scalar.',
+                      caller=sys._getframe(3).f_code.co_name)
+
+    if prop in COLOR_PROPERTIES:
+        return verify_hex_colors(none_check(prop_val))
+    elif prop in DIMENSION_PROPERTIES:
+        return verify_dimensions(DIMENSION_PROPERTIES[prop], none_check(prop_val))
+    elif prop in OPACITY_PROPERTIES:
+        return verify_opacities(none_check(prop_val))
+    elif prop in SHAPE_PROPERTIES:
+        return verify_node_shapes(none_check(prop_val), styles.get_node_shapes())
+    elif prop in LINE_STYLE_PROPERTIES:
+        return verify_edge_shapes(none_check(prop_val), styles.get_line_styles(), 'line style', 'get_line_style')
+    elif prop in ARROW_STYLE_PROPERTIES:
+        return verify_edge_shapes(none_check(prop_val), styles.get_arrow_shapes(), 'arrow style', 'get_arrow_shapes')
+    elif prop in VISIBLE_PROPERTIES:
+        return verify_bools(none_check(prop_val))
+    elif prop in LABEL_PROPERTIES | TOOLTIP_PROPERTIES | FONT_FACE_PROPERTIES:
+        return verify_strs(none_check(prop_val))
+    elif prop in CUSTOM_GRAPHICS_PROPERTIES:
+        return prop_val
+    else:
+        show_error(f'Warning: setting unknown property "{prop}" to "{prop_val}"')
+        return prop_val
+
