@@ -37,7 +37,6 @@ from . import network_views
 from .exceptions import CyError
 from .py4cytoscape_utils import *
 from .py4cytoscape_logger import cy_log
-from .py4cytoscape_notebook import running_remote
 from .py4cytoscape_sandbox import get_abs_sandbox_path
 
 # ==============================================================================
@@ -84,6 +83,42 @@ def copy_visual_style(from_style, to_style, base_url=DEFAULT_BASE_URL):
     res = commands.cyrest_put(f'styles/{to_style}/dependencies', body=res, base_url=base_url, require_json=False)
     return res
 
+@cy_log
+def get_visual_style_JSON(style_name, css=False, base_url=DEFAULT_BASE_URL):
+    """Get all defaults and mappings for a visual style
+
+    Args:
+        style_name (str): name of style
+        css (bool): True to create a CytoscapeJS CSS style, False to create a generic JSON version
+        base_url (str): Ignore unless you need to specify a custom domain,
+            port or version to connect to the CyREST API. Default is http://127.0.0.1:1234
+            and the latest version of the CyREST API supported by this version of py4cytoscape.
+
+    Returns:
+        dict or list: dict if generic JSON, list if CSS
+
+    Raises:
+        CyError: if empty style name
+        requests.exceptions.RequestException: if can't connect to Cytoscape or Cytoscape returns an error
+
+    Examples:
+        >>> get_visual_style_JSON('galFiltered Style')
+        {'title': 'galFiltered Style', 'defaults': [{'visualProperty': 'COMPOUND_NODE_PADDING', 'value': 10.0}...]}
+        >>> get_visual_style_JSON('galFiltered Style', css=True)
+        [{'format_version': '1.0', 'generated_by': 'cytoscape-3.9.1', 'target_cytoscapejs_version': '~2.1'...}]
+    """
+    if style_name == None or len(style_name.strip()) == 0:
+        raise CyError(f'No style name provided')
+
+    if css:
+        style_name += '.json'   # get CSS-style JSON for CytoscapeJS usage
+    else:
+        pass # this is the default ... leave style_name alone
+
+    res = commands.cyrest_get(f'styles/{style_name}', base_url=base_url)
+    return res
+
+
 @cy_log
 def create_visual_style(style_name, defaults=None, mappings=None, base_url=DEFAULT_BASE_URL):
     """Create a style from defaults and predefined mappings.
@@ -92,7 +127,7 @@ def create_visual_style(style_name, defaults=None, mappings=None, base_url=DEFAU
 
     Args:
         style_name (str): name for style
-        defaults (list): key-value pairs for default mappings.
+        defaults (dict or list): for dict, key-value pairs for default mappings. for list, [{'visualProperty': prop-name, 'value': default-val}, ...]
         mappings (list): visual property mappings, see ``map_visual_property``
         base_url (str): Ignore unless you need to specify a custom domain,
             port or version to connect to the CyREST API. Default is http://127.0.0.1:1234
@@ -113,16 +148,24 @@ def create_visual_style(style_name, defaults=None, mappings=None, base_url=DEFAU
         >>> edge_width = map_visual_property('edge width', 'EdgeBetweenness', 'p')
         >>> create_visual_style('NewStyle', defaults=defaults, mappings=[node_labels, node_fills, arrow_shapes, edge_width])
         {'title': 'NewStyle'}
+        >>> defaults = [{'visualProperty': 'NODE_SHAPE', 'value': 'diamond'}, {'visualProperty': 'NODE_SIZE', 'value': 30}]
+        >>> create_visual_style('NewStyle1', defaults=defaults)
+        {'title': 'NewStyle1'}
 
     Note:
         To apply the style to a network, first create the network and then call ``set_visual_style``
 
+        This function can be used directly with the ``defaults`` and ``mappings`` members returned by ``get_visual_style_JSON``
+
     See Also:
-        :meth:`map_visual_property`, :meth:`set_visual_style`
+        :meth:`map_visual_property`, :meth:`set_visual_style`, :meth:`get_visual_style_JSON`
     """
     if mappings is None: mappings = []
-    style_def = []
-    if defaults is not None:
+    if defaults is None:
+        style_def = []
+    elif isinstance(defaults, list):
+        style_def = defaults # already in [{'visualProperty':xxx, 'value':yyy}, ... ] format
+    else:
         style_def = [{'visualProperty': key, 'value': val}  for key, val in defaults.items()]
     style = {'title': style_name, 'defaults': style_def, 'mappings': mappings}
     res = commands.cyrest_post('styles', body=style, base_url=base_url)
@@ -152,6 +195,30 @@ def delete_visual_style(style_name, base_url=DEFAULT_BASE_URL):
     res = commands.cyrest_delete(f'styles/{style_name}', base_url=base_url, require_json=False)
     return res
 
+@cy_log
+def delete_all_visual_styles(base_url=DEFAULT_BASE_URL):
+    """Delete all visual styles from current Cytoscape session.
+
+    The 'default' style remains after all other styles are deleted.
+
+    Args:
+        base_url (str): Ignore unless you need to specify a custom domain,
+            port or version to connect to the CyREST API. Default is http://127.0.0.1:1234
+            and the latest version of the CyREST API supported by this version of py4cytoscape.
+
+    Returns:
+        str: ''
+
+    Raises:
+        requests.exceptions.RequestException: if can't connect to Cytoscape or Cytoscape returns an error
+
+    Examples:
+        >>> delete_all_visual_styles()
+        ''
+    """
+    res = commands.cyrest_delete(f'styles', base_url=base_url, require_json=False)
+    return res
+
 @cy_log
 def export_visual_styles(filename=None, type='XML', styles=None, base_url=DEFAULT_BASE_URL, *, overwrite_file=False):
     """Save one or more visual styles to file.
