@@ -3,6 +3,7 @@
 """Functions for managing TABLE columns and table column functions, like map and rename, as well as loading and
 extracting table data in Cytoscape.
 """
+import sys
 
 """Copyright 2020-2022 The Cytoscape Consortium
 
@@ -125,14 +126,9 @@ def get_table_columns(table='node', columns=None, namespace='default', network=N
         4608       0                     1
         4609    2092                     2
         ...
-        >>> get_table_columns(columns='Stress, bogus')
-              Stress bogus
-        4608       0   NaN
-        4609    2092   NaN
-        ...
 
     Note:
-        For requested columns not present in the table, the column is still returned but is full of ``nan`` values.
+        For requested columns not present in the table, the column is not returned, but a warning is shown.
     """
     suid = networks.get_network_suid(network, base_url)
 
@@ -151,7 +147,7 @@ def get_table_columns(table='node', columns=None, namespace='default', network=N
     # get suid column first and make a dataframe with SUID as index
     res_names = commands.cyrest_get(f'networks/{suid}/tables/{namespace}{table}/columns/SUID', base_url=base_url)
     suid_list = res_names['values']
-    df = pd.DataFrame(index=suid_list, columns=col_list)
+    df = pd.DataFrame(index=suid_list)
 
     # then fill in each requested column
     for col in col_list:
@@ -186,9 +182,9 @@ def get_table_columns(table='node', columns=None, namespace='default', network=N
         if len(suid_list) != len(cvv):
             narrate('Column "%s" has only %d elements, but should have %d' % (col, len(cvv), len(suid_list)))
             break  # TODO: Is this the right response?
-        # TODO: Consider assigning entire column all at once instead of iterating ... should be OK
-        for val, suid_val in zip(cvv, suid_list):
-            df.at[suid_val, col] = val
+
+        # Assign entire column, assuming values are ordered consistently by Cytoscape
+        df[col] = cvv
 
     return df
 
@@ -420,6 +416,7 @@ def load_table_data(data, data_key_column='row.names', table='node', table_key_c
         data (dataframe): each row is a node and columns contain node attributes
         data_key_column (str): name of data.frame column to use as key; ' default is "row.names"
         table (str): name of Cytoscape table to load data into, e.g., node, edge or network; default is "node"
+        table_key_column (str): name of Cytoscape table column to use as key; default is "name"
         namespace (str): Namespace of table. Default is "default".
         network (SUID or str or None): Name or SUID of a network. Default is the
             "current" network active in Cytoscape.
@@ -443,6 +440,9 @@ def load_table_data(data, data_key_column='row.names', table='node', table_key_c
         >>> load_table_data(data, data_key_column='id', table='node', table_key_column='name', network='galfiltered.sif')
         'Success: Data loaded in defaultnode table'
     """
+    if type(table_key_column) is not str:
+        raise CyError('table_key_column must be the name of a single column.')
+
     net_suid = networks.get_network_suid(network, base_url=base_url)
     table_key_column_values = get_table_columns(table=table, namespace=namespace, columns=table_key_column,
                                                 network=net_suid, base_url=base_url)
@@ -457,15 +457,26 @@ def load_table_data(data, data_key_column='row.names', table='node', table_key_c
     if not data_key_column in data.columns:
         raise CyError('Failed to load data. Please check data_key_column.')
 
+    # create table containing columns present in data and already present in Cytoscape table and
     # verify that there is at least one key in the Cytoscape table that matches a key in the data
-    table_keys = table_key_column_values[table_key_column].astype(str).values
-    filter = [str(key) in table_keys     for key in data[data_key_column]]
-    if not True in filter:
+    # Note: we rely on isin() to do the proper type comparison for the data values involved
+    data_subset = data[data[data_key_column].isin(table_key_column_values[table_key_column])]
+    if data_subset.empty:
+        from .py4cytoscape_logger import detail_logger
+        detail_logger.debug(f'Pandas: {pd.__version__}')
+        detail_logger.debug(f'Python: {sys.version}')
+        detail_logger.debug(f'data_key_column: {data_key_column}')
+        detail_logger.debug(f'table_key_column: {table_key_column}')
+        detail_logger.debug(f'data: {data}')
+        detail_logger.debug(f'type(data[data_key_column]): {type(data[data_key_column])}')
+        detail_logger.debug(f'data[data_key_column]: {data[data_key_column]}')
+        detail_logger.debug(f'table_key_column_values: {table_key_column_values}')
+        detail_logger.debug(
+            f'type(table_key_column_values[table_key_column]): {type(table_key_column_values[table_key_column])}')
+        detail_logger.debug(f'table_key_column_values[table_key_column]: {table_key_column_values[table_key_column]}')
+        detail_logger.debug(f'data_subset: {data_subset}')
         raise CyError(f'Provided table key column "{table_key_column}" and data key column "{data_key_column}" do not contain any matches')
 
-    # create table containing columns present in data and already present in Cytoscape table
-    data_subset = data[filter]
-
     # look for elements that are lists (instead of scalars) and turn them into comma-separated strings.
     # Note that CyREST doesn't accept lists or create columns of type list, but comma-separated strings is
     # the best we can do for the user at this time.
@@ -487,8 +498,7 @@ def load_table_data(data, data_key_column='row.names', table='node', table_key_c
                                     body={'name': x, 'type': 'Integer'}, require_json=False, base_url=base_url)
 
     existing_cols = get_table_column_names(table, namespace, net_suid, base_url=base_url)
-    [create_col(x[0]) if x[1] == 'int64' and not x[0] in existing_cols else None for x in
-     data_subset.dtypes.iteritems()]
+    [create_col(x[0]) if x[1] == 'int64' and not x[0] in existing_cols else None     for x in data_subset.dtypes.items()]
 
     # finally, add the values for whatever columns we have (and create new columns as needed)
     res = commands.cyrest_put(f'networks/{net_suid}/tables/{tbl}',
